module DockerRailsProxy
  class Kubectl < Base
    autoload :Bash,          'docker_rails_proxy/commands/kubectl/bash'
    autoload :SetKubeconfig, 'docker_rails_proxy/commands/kubectl/set_kubeconfig'

    KUBECONFIG_PATH = build_path('tmp/.kube/kubeconfig')

    before_initialize do
      'kubectl is required, `brew install kubectl`' unless system 'type kubectl &> /dev/null'
    end

    validates do
      unless File.exist?(KUBECONFIG_PATH)
        <<-EOS

        kubeconfig is required, run this command to set it
        `bin/#{APP_NAME} kubectl set-kubeconfig --path /path/kubeconfig`
        EOS
      end unless arguments.first == 'set-kubeconfig'
    end

    builds ->(params:) do
      case params[:arguments].first
      when 'bash'           then Bash
      when 'set-kubeconfig' then SetKubeconfig
      end
    end

    def process
      kubectl arguments.join(' ')
    end

    private

    def kubectl(command)
      cmd =  "kubectl --kubeconfig='#{KUBECONFIG_PATH}' #{command}"
      cmd << " -- #{additional_arguments.join(', ')}" unless additional_arguments.empty?
      exec cmd
    end

    def kubectl_output(command)
      %x(kubectl --kubeconfig='#{KUBECONFIG_PATH}' #{command}).strip
    end
  end
end
