require 'fileutils'
require 'yaml'

module DockerRailsProxy
  class Kubectl < Base
    class SetKubeconfig < self
      attr_accessor :options, :data

      after_initialize { self.options = {} }
      after_initialize { opt_parser.parse!(arguments) }

      validates do
        if options[:path].nil?
          '--path is required'
        elsif !File.exist?(options[:path])
          "#{options[:path]} kubeconfig does not exist."
        end
      end

      validates do
        self.data = YAML.load_file(options[:path])

        unless data['kind'] == 'Config'
          "#{options[:path]} is not a valid kubeconfig file"
        end
      end

      before_process do
        FileUtils.mkdir_p 'tmp/.kube'

        absolute_dir = File.dirname(options[:path])

        data['clusters'].each do |hash|
          cluster = hash['cluster']

          if cluster['certificate-authority'].present?
            cluster['certificate-authority'] = File.absolute_path(
              cluster['certificate-authority'], absolute_dir
            )
          end
        end

        data['users'].each do |hash|
          user = hash['user']

          if user['client-certificate'].present?
            user['client-certificate'] = File.absolute_path(
              user['client-certificate'], absolute_dir
            )
          end

          if user['client-key'].present?
            user['client-key'] = File.absolute_path(
              user['client-key'], absolute_dir
            )
          end
        end
      end

      def process
        File.write(KUBECONFIG_PATH, data.to_yaml)
        puts 'kubeconfig was set correctly'
      end

    private

      def opt_parser
        @opt_parser ||= OptionParser.new do |opts|
          opts.banner = "Usage: bin/#{APP_NAME} kubectl #{self.class.name.demodulize.parameterize} [options]"

          opts.on('--path PATH', 'kubeconfig file path') do |path|
            options[:path] = path
          end

          yield opts if block_given?

          opts.on('-h', '--help', 'Display this screen') do
            puts opts
            exit
          end
        end
      end
    end
  end
end
