module DockerRailsProxy
  class Rails < SyncBack
    attr_reader :args

    autoload :Credentials, 'docker_rails_proxy/commands/rails/credentials'

    builds ->(params:) do
      case params[:arguments].first
      when 'credentials' then Credentials
      else
        DockerRailsProxy::Rails
      end
    end

    def process
      command, *@args = arguments

      case command
      when 'c',  'console'    then console
      when 'db', 'dbconsole'  then db
      when 'logs'             then logs
      when 'restart', 'touch' then restart
      when 'secrets'          then secrets
      else
        execute "bin/rails #{command} #{args.join(' ')}", tty: true
      end
    end

    private

    def console
      execute "bin/rails c #{args.join(' ')}", tty: true, replace_process: true
    end

    def db
      container_id = get_docker_container_id(:mysql)

      if container_id.empty?
        logger.error "Couldn't connect to mysql container, make sure it's running"
        exit 1
      end

      execute(
        "mysql #{args.first || "#{APP_NAME}_development"}",
        container_id: container_id,
        tty: true,
        replace_process: true
      )
    end

    def logs
      execute "tail -f log/#{args.first || 'development'}.log", replace_process: true
    end

    def restart
      execute "touch tmp/restart.txt", replace_process: true
    end

    def secrets
      execute "bash -c 'EDITOR=vim bin/rails secrets:edit'", tty: true
    end

    def credentials
      # Temporary solution to use credentials in Rails 5.2
      # Future version will use the --environment argument instead
      command = "RAILS_ENV=#{args.first} EDITOR=vim bin/rails credentials:edit"
      execute "bash -c '#{command}'", tty: true
    end
  end
end
